<?php

namespace Drupal\default_content_deploy\Normalizer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\default_content_deploy\Form\SettingsForm;
use Drupal\hal\LinkManager\LinkManagerInterface;
use Drupal\hal\Normalizer\ContentEntityNormalizer;

/**
 * Configurable Normalizer for DCD edge-cases.
 */
class ConfigurableContentEntityNormalizer extends ContentEntityNormalizer {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * Constructs a ContentEntityNormalizer object.
   *
   * @param \Drupal\hal\LinkManager\LinkManagerInterface $link_manager
   *   The hypermedia link manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Entity\EntityTypeRepositoryInterface $entity_type_repository
   *   The entity type repository.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *    The config factory.
   */
  public function __construct(LinkManagerInterface $link_manager, EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, EntityTypeRepositoryInterface $entity_type_repository, EntityFieldManagerInterface $entity_field_manager, ConfigFactoryInterface $config) {
    parent::__construct($link_manager, $entity_type_manager, $module_handler, $entity_type_repository, $entity_field_manager);
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($entity, $format = NULL, array $context = array()) : float|array|int|bool|\ArrayObject|string|null  {
    $config = $this->config->get(SettingsForm::DIRECTORY);
    if ($config->get('skip_computed_fields') ?? FALSE) {
      // Check if the entity has computed fields and remove them.
      foreach ($entity->getFieldDefinitions() as $field_name => $field_definition) {
        if (isset($entity->$field_name) && $field_definition->isComputed()) {
          unset($entity->$field_name);
        }
      }
    }

    return parent::normalize($entity, $format, $context);
  }

}
